# Config show dialog
import json
import logging
from nicegui import app, ui
from ezlab.parameters import DF, UA
from ezlab.pages import ezmeral

from ezlab.pages.sshkeys import ssh_keys_ui
from ezlab.pages import targets
from ezlab.pages import settings
from ezlab.utils import CONFIG_DL_PATH

logger = logging.getLogger("ezlab.ui.settings")


def save_config(val: str, dialog):
    try:
        for key, value in json.loads(val.replace("\n", "")).items():
            app.storage.general[key] = value
        for m in (ezmeral.menu, targets.menu, settings.menu):
            m.refresh()
        dialog.close()
        ui.notify("Settings loaded", type="positive")
    except (TypeError, json.decoder.JSONDecodeError, ValueError) as error:
        ui.notify("Not a valid json", type="negative")
        logger.warning(error)


def config_show():
    with ui.dialog() as config_show, ui.card().classes("grow"):
        ui.code(json.dumps(app.storage.general, indent=2), language="json").classes("w-full text-wrap")
        with ui.row().classes("w-full"):
            ui.button(
                "Save",
                icon="save",
                on_click=lambda: ui.download(CONFIG_DL_PATH),
            )
            ui.space()
            ui.button(
                "Close",
                icon="cancel",
                on_click=config_show.close,
            )
    return config_show


# Config load dialog
def config_load():
    with ui.dialog() as config_load, ui.card().classes("grow"):
        ui.upload(label="Config JSON", auto_upload=True, on_upload=lambda e: save_config(e.content.read().decode("utf-8"), config_load)).classes(
            "max-w-full"
        )
        # jsontext = ui.textarea().props("stack-label=json autogrow filled").classes("h-dvh w-full text-wrap")
        # with ui.row():
        #     ui.button(
        #         "Save",
        #         on_click=lambda _: save_config(
        #             jsontext.value,
        #             config_load,
        #         ),
        #         icon="save",
        #     )
        #     ui.button(
        #         "Close",
        #         icon="cancel",
        #         on_click=config_load.close,
        #     )

    return config_load


@ui.refreshable
def menu():
    with (
        ui.expansion(
            "Settings",
            icon="settings",
            caption="for your environment",
        )
        .classes("w-full")
        .classes("text-bold") as settings
    ):
        with ui.row().classes("w-full place-items-end"):
            ui.input("Datacenter").bind_value(app.storage.general, "datacenter").classes("grow")
            # ui.space()
            ui.checkbox("Dry Run", value=False).bind_value(app.storage.general["config"], "dryrun")
        ui.label("Network Settings").classes("text-bold")
        with ui.row().classes("w-full"):
            ui.input("VM Network", placeholder="10.1.1.0/24").bind_value(app.storage.general["config"], "cidr").classes("grow")
            ui.input("Gateway", placeholder="10.1.1.1").bind_value(app.storage.general["config"], "gateway").classes("grow")
            ui.input("HTTP Proxy", placeholder="").bind_value(app.storage.general["config"], "proxy").classes("grow")
        with ui.row().classes("w-full"):
            ui.input("Name Server", placeholder="10.1.1.1").bind_value(app.storage.general["config"], "nameserver").classes("grow")
            ui.space()
            ui.input("Domain", placeholder="ez.lab").bind_value(app.storage.general["config"], "domain").classes("grow")

        ui.label("Repositories").classes("text-bold")
        with ui.row().classes("w-full place-items-end"):
            # YUM Repo
            ui.input("YUM Repo", placeholder="").bind_value(app.storage.general["config"], "yumrepo").classes("grow")
            ui.input("EPEL Repo", placeholder="").bind_value(app.storage.general["config"], "epelrepo").classes("grow")
            # Airgap registry settings
            ui.input("Registry URL", placeholder="").bind_value(app.storage.general[UA], "registryUrl").classes("grow")
        # with ui.row().classes("w-full"):
        # ui.checkbox("Registry Insecure").bind_value(app.storage.general[UA], "registryInsecure")
        # ui.input("Registry User", placeholder="").bind_value(app.storage.general[UA], "registryUsername")
        # ui.input("Registry Password", placeholder="").bind_value(app.storage.general[UA], "registryPassword")
        # ui.input("Registry CA File (base64)", placeholder="").bind_value(app.storage.general[UA], "registryCaFile")

        # MapR Repository Configuration
        with ui.row().classes("w-full place-items-end"):
            ui.label("MapR Repo").classes("self-center")
            switch = ui.switch("Use local", value=False).props("left-label").bind_value(app.storage.general[DF], "maprrepoislocal")

            # Using default HPE repository
            with ui.row().bind_visibility_from(switch, "value", lambda x: not x).classes("w-full"):
                ui.input("HPE Passport e-mail").bind_value(app.storage.general[DF], "maprrepouser").classes("grow")
                ui.input("HPE Passport token", password=True, password_toggle_button=True).bind_value(
                    app.storage.general[DF], "maprrepotoken"
                ).classes("grow")

            # Using local repository
            with ui.row().classes("w-full place-items-end").bind_visibility_from(switch, "value"):
                ui.input(
                    "MapR Repo",
                    placeholder="https://repo.ez.lab/mapr/",
                ).bind_value(
                    app.storage.general[DF], "maprlocalrepo"
                ).classes("grow")
                authlocal = ui.switch("Authenticate", value=True).bind_value(app.storage.general[DF], "maprlocalrepoauth")
                with ui.row().classes("w-full place-items-end"):
                    ui.input("Username", password=True).bind_value(app.storage.general[DF], "maprlocalrepousername").bind_visibility(
                        authlocal, "value"
                    ).classes("grow")
                    ui.input("Password", password=True).bind_value(app.storage.general[DF], "maprlocalrepopassword").bind_visibility(
                        authlocal, "value"
                    ).classes("grow")

        ui.label("Cloudinit Settings").classes("text-bold")
        with ui.row().classes("w-full place-items-end"):
            ui.input("username", placeholder="ezmeral").bind_value(app.storage.general["config"], "username").classes("grow")
            ui.input("password", password=True, password_toggle_button=True).bind_value(app.storage.general["config"], "password").classes("grow")

        ui.label("SSH Key").classes("text-bold")
        ssh_keys_ui()

    settings.bind_value(app.storage.general["ui"], "settings")



import logging
from nicegui import app, ui, run
from ezlab.pages.df import get_doc
from ezlab.parameters import EZNODES, PVE
from ezinfra.vms import clone
from ezinfra import pve


logger = logging.getLogger("ezlab.ui.vms")


def pve_newvm_dialog():

    # @ui.refreshable
    # def bridge_selector():
    #     app.storage.user["busy"] = True
    #     selector = (
    #         ui.select(
    #             options=app.storage.user.get("bridges", []),
    #             label="Base Image",
    #         )
    #         .classes("w-full")
    #         .props("inline")
    #         .bind_value(app.storage.general[PVE], "bridge")
    #     )
    #     app.storage.user["busy"] = False
    #     return selector

    # def update_template():
    #     app.storage.user["bridges"] = pve.bridges(app.storage.general[PVE].get("template", None))
    #     bridge_selector.refresh()

    app.storage.user["busy"] = True

    templates = [x for x in pve.vms() if x["template"] and x["type"] == "qemu"]

    storage = pve.pools()

    app.storage.user["busy"] = False

    # vm settings dialog
    with ui.dialog() as dialog, ui.card():
        templateid = (
            ui.select(
                options={x["id"]: f"{x['name']} ( {x['node']} )" for x in templates},
                label="Template",
                # on_change=update_template,
            )
            .classes("w-full")
            .props("inline")
            .bind_value(app.storage.general[PVE], "template")
        )

        ui.select(
            options={x["id"]: f"{x['storage']} ( {x['node']} )" for x in storage},
            # if x["node"] == app.storage.user["template"]["node"]
            label="Data Disk Pool",
        ).classes("w-full").props("inline").bind_value(app.storage.general[PVE], "storage")

        # network_selector = (
        #     ui.select(
        #         options=[
        #             x["sdn"]
        #             for x in networks
        #             # if x["node"] == app.storage.user["template"]["node"]
        #         ],
        #         label="Network",
        #     )
        #     .classes("w-full")
        #     .props("inline")
        #     .bind_value(app.storage.general[PVE], "network")
        # )

        ui.select(
            # options=[x["iface"] for x in app.storage.user.get("bridges", [])],
            options=[x["iface"] for x in pve.bridges(templateid.value)],
            label="Bridge",
        ).classes("w-full").props("inline").bind_value(app.storage.general[PVE], "bridge")

        ui.select(
            options=[x["name"] for x in EZNODES],
            label="Node Type",
        ).classes("w-full").props(
            "inline"
        ).bind_value(app.storage.general[PVE], "eznode")

        ui.input(label="VM Name", placeholder="ezvm").classes("w-full").props("inline").bind_value(app.storage.general[PVE], "hostname")

        ui.input(
            label="First IP",
            placeholder=app.storage.general["config"]["gateway"],
        ).classes("w-full").props(
            "inline"
        ).bind_value(app.storage.general[PVE], "firstip")

        ui.button(
            "Create",
            on_click=lambda: dialog.submit(
                (
                    next(
                        (i for i in templates if i["id"] == app.storage.general[PVE]["template"]),
                        None,
                    ),
                    next(
                        (i for i in storage if i["id"] == app.storage.general[PVE]["storage"]),
                        None,
                    ),
                    app.storage.general[PVE]["bridge"],
                    next(
                        (i for i in EZNODES if i["name"] == app.storage.general[PVE]["eznode"]),
                        None,
                    ),
                    app.storage.general[PVE]["hostname"],
                    app.storage.general[PVE]["firstip"],
                )
            ),
        )

    return dialog


async def new_vm_ui():

    if app.storage.general["target"]["hve"] == PVE:
        dialog = pve_newvm_dialog()
    else:
        ui.notify("not implemented")

    resources = await dialog

    if resources and all(resources):
        vm_count = resources[3]["count"]

        app.storage.user["busy"] = True
        dryrun = app.storage.general["config"].get("dryrun", True)

        result = await run.io_bound(
            clone,
            target=app.storage.general["target"]["hve"],
            resources=resources,
            settings=dict(app.storage.general["config"]),
            # vm_number=(0 if vmcount == 1 else count + 1),  # start from 1 not 0, use 0 for single vm
            vm_count=vm_count,
            dryrun=dryrun,
        )
        if dryrun:
            get_doc(result)
        elif result:
            try:
                for res in result:
                    logger.info("Got notification: %s", res)
            except:
                logger.debug("Not a generator")
                logger.info("%s VMs ready: %s", vm_count, result)

        app.storage.user["busy"] = False

    else:
        ui.notify("VM creation cancelled", type="info")
